<?php

namespace Service;

use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Environment;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\ErrorRepository as ErrorRepEntity;
use Factory\ErrorRepository as ErrorRepositoryFactory;

class ErrorRepository {
    /** @var  Environment $environment */
    protected $environment;
    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var ErrorRepositoryFactory $errorRepositoryFactory */
    protected $errorRepositoryFactory;

    public function __construct(Environment $environment, EntityManager $entityManager, ErrorRepositoryFactory $errorRepositoryFactory) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->errorRepositoryFactory = $errorRepositoryFactory;
    }

    /**
     * @param $conditions
     * @param $page
     * @param $pageSize
     * @return array
     */
    public function findErrorRepository($conditions, $page, $pageSize) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filters = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getErrorRepository()->findErrorRepository($filters, $limit, $offset);
    }

    /**
     * @param $id
     * @param $reason
     * @param $resolution
     * @param $description
     * @return array
     * @throws \Doctrine\ORM\ORMException
     */
    public function saveErrorRepository($id, $reason, $resolution, $description, $type) {
        /** @var ErrorRepEntity $errorRepository */
        $errorRepository = InstanceProvider::getRepository()->getErrorRepository()->find($id);
        $errorRep = $this->errorRepositoryFactory->saveErrorRepository($errorRepository, $description, $reason, $resolution, $type);
        $this->entityManager->persist($errorRep);
        $this->entityManager->flush();
        return array('ID' => $errorRep->getId());
    }

    /**
     * @param $id
     * @throws \Doctrine\ORM\ORMException
     */
    public function deleteError($id) {
        $errorRepository = InstanceProvider::getRepository()->getErrorRepository()->find($id);
        $this->entityManager->remove($errorRepository);
        $this->entityManager->flush();
    }
}