<?php
namespace Service;

use Util\Messages;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Util\Utilities as LibUtilities;
use Zeedhi\DevOps\Model\Profile\Entities\ZhUser;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class InitService {
    const ID_CLIENTE_LIBRARIES = "cd7e85178153by60953518945f9hakt19a50a4";
    const SECRET_CLIENTE_LIBRARIES = "lib5449996c21f780fe30badd6521931a5sac9";

    protected $pmUrl;
    protected $projectId;

    /** @var Session $sessionManager */
    protected $sessionManager;
    /** @var Environment $environment */
    protected $environment;
    /** @var \Zeedhi\DevOps\Model\Profile\Repositories\ZhUser */
    protected $zhUserRepository;
    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource */
    protected $resourceRepository;

    public function __construct($sessionManager, Environment $environment, EntityManager $entityManager,
                                LibUtilities $libUtilities, $pmUrl, $projectId){
		$this->sessionManager = $sessionManager;
		$this->environment = $environment;
		$this->entityManager = $entityManager;
		$this->libUtilities = $libUtilities;
		$this->pmUrl = $pmUrl;
		$this->projectId = $projectId;

        $this->resourceRepository = $this->entityManager->getRepository(Utilities::PROJECT_MANAGEMENT_ENTITY_DIR . 'Resource');
        $this->zhUserRepository   = $this->entityManager->getRepository(Utilities::PROFILE_ENTITY_DIR. 'ZhUser');
	}

    /**
     * @throws \Exception
     * @return array
     */
    public function initSession() {
        /** @var Entities\Resource $resource */
        $resource = $this->resourceRepository->findOneBy(array('externalCode' => $this->environment->getCurrentCdoperador()));
        if(empty($resource)) {
            throw new \Exception(Messages::ERR_LOGGED_USER_NOT_FOUND);
        } else {
            $resourceId = $resource->getId();
            $this->sessionManager->set("organizationId", $resource->getOrganization());
            $this->sessionManager->set("resourceId", $resourceId);

            $userId = $this->validateZhUser($resourceId);
            $this->sessionManager->set("userId", $userId);

            $ds = DIRECTORY_SEPARATOR;
            $baseDirImages = $this->environment->getAttachmentsBaseDir() . $ds . 'profile';
            $photoPath = $baseDirImages . $ds . $resourceId;
            $photo = null;

            if (file_exists($photoPath)) {
                $photo = file_get_contents($photoPath);
            }

            return array(
                'PHOTO' => $photo,
                'RESOURCE_ID' => $resourceId,
                'ZH_USER_ID' => $userId,
                'LOGIN_TOKEN' => $this->environment->getLoginTokenInSession()
            );
        }
	}

    /**
     * @param $resourceId
     * @throws \Exception
     * @return bool|int
     */
    private function validateZhUser($resourceId) {
        /** @var ZhUser $zhUser */
//        $zhUser = $this->zhUserRepository->findOneBy(array('externalId' => $resourceId));
        $zhUser = $this->zhUserRepository->findOneBy(array('externalResId' => $resourceId));
        if(empty($zhUser)) {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            try {
                /** @var ZhUser $defaultUser */
                $defaultUser = $this->zhUserRepository->find(1);

                $user = new ZhUser();
                $user->setName($this->sessionManager->get('USER_NAME'));
                $user->setEmail($this->sessionManager->get('EMAIL'));
//                $user->setExternalId($resourceId);
                $user->setExternalResId($resourceId);
                $user->setCreatedBy($defaultUser);
                $user->setUpdatedBy($defaultUser);
                $user->setCreated(new \DateTime());

                $this->entityManager->persist($user);
                $this->entityManager->flush();

                $connection->commit();
                $userId = $user->getId();
            } catch(\Exception $e) {
                $connection->rollback();
                throw $e;
            }
        } else {
            $userId = $zhUser->getId();
        }
        return $userId;
    }

    public function getTokenPM() {
        if($this->environment->getTokenPM()) {
            return $this->environment->getTokenPM();
        } else {
            $param = array(
                'CLIENT_ID' => self::ID_CLIENTE_LIBRARIES,
                'CLIENT_SECRET' => self::SECRET_CLIENTE_LIBRARIES
            );
            $token = $this->libUtilities->callWebService(null, '/lib_requestToken', $param, 'token',
                self::SECRET_CLIENTE_LIBRARIES, $this->pmUrl);
            $this->environment->setTokenPM($token[0]);
            return $token[0];
        }
    }
}
